@@ -86,7 +86,7 @@ class FortranAssignOp(PyRatesAssignOp):
         ndim = self._op_dict['args'][idx].shape
         return generate_func(self, return_key=return_key, omit_assign=True, return_dim=ndim, return_intent='out')
 
-    def eval(self):
+    def numpy(self):
         result = self._callable(*self.args[1:])
         self._check_numerics(result, self.name)
         return result
@@ -207,8 +207,6 @@ class FortranBackend(NumpyBackend):
         # preparations
         ##############
 
-        os.chdir(self._build_dir)
-
         # remove empty layers and operators
         new_layer_idx = 0
         for layer_idx, layer in enumerate(self.layers.copy()):
@@ -355,16 +353,18 @@ class FortranBackend(NumpyBackend):
         func_gen.remove_indent()
 
         # save rhs function to file
-        f2py.compile(func_gen.generate(), modulename='rhs_func', extension='.f', source_fn='rhs_func.f', verbose=False)
+        fname = f'{self._build_dir}/rhs_func'
+        f2py.compile(func_gen.generate(), modulename='rhs_func', extension='.f', source_fn=f'{fname}.f', verbose=False)
 
         # create additional subroutines in pyauto compatibility mode
-        if self.pyauto_compat:
+        gen_def = kwargs.pop('generate_auto_def', True)
+        if self.pyauto_compat and gen_def:
             self.generate_auto_def(self._build_dir)
 
         # import function from file
-        exec("from rhs_func import func", globals())
+        fname_import = 'func' if self.pyauto_compat else 'rhs_eval'
+        exec(f"from rhs_func import {fname_import}", globals())
         rhs_eval = globals().pop('func')
-        os.chdir(self._orig_dir)
 
         # apply function decorator
         if decorator:
@@ -394,7 +394,7 @@ class FortranBackend(NumpyBackend):
         try:
 
             # read file from excisting system compilation
-            if not directory:
+            if directory is None:
                 directory = self._build_dir
             fn = f"{directory}/rhs_func.f" if "rhs_func.f" not in directory else directory
             with open(fn, 'rt') as f:
@@ -403,7 +403,7 @@ class FortranBackend(NumpyBackend):
         except FileNotFoundError:
 
             # compile system and then read the build files
-            self.compile(build_dir=directory)
+            self.compile(build_dir=directory, generate_auto_def=False)
             fn = f"{self._build_dir}/rhs_func.f"
             with open(fn, 'r') as f:
                 func_str = f.read()
@@ -483,7 +483,7 @@ class FortranBackend(NumpyBackend):
         for key, (vtype, idx) in var_map.items():
             var = self.get_var(key)
             if vtype == 'state_var':
-                func_gen.add_code_line(f"{var.value} = {var.eval()}")
+                func_gen.add_code_line(f"{var.value} = {var.numpy()}")
                 func_gen.add_linebreak()
         func_gen.add_linebreak()
 
@@ -512,10 +512,11 @@ class FortranBackend(NumpyBackend):
         #########################
 
         # declare auto constants and their values
-        auto_constants = {'NDIM': self.ndim, 'NPAR': self.npar, 'IPS': -2, 'ILP': 0, 'ICP': 14, 'NTST': 1, 'NCOL': 4,
-                          'IAD': 3, 'ISP': 0, 'ISW': 1, 'IPLT': 0, 'NBC': 0, 'NINT': 0, 'NMX': 10000, 'NPR': 10,
+        auto_constants = {'NDIM': self.ndim, 'NPAR': self.npar, 'IPS': -2, 'ILP': 0, 'ICP': [14], 'NTST': 1, 'NCOL': 4,
+                          'IAD': 3, 'ISP': 0, 'ISW': 1, 'IPLT': 0, 'NBC': 0, 'NINT': 0, 'NMX': 10000, 'NPR': 100,
                           'MXBF': 10, 'IID': 2, 'ITMX': 8, 'ITNW': 5, 'NWTN': 3, 'JAC': 0, 'EPSL': 1e-7, 'EPSU': 1e-7,
-                          'EPSS': 1e-5, 'IRS': 0}
+                          'EPSS': 1e-5, 'IRS': 0, 'DS': 1e-4, 'DSMIN': 1e-8, 'DSMAX': 1e-2, 'IADS': 1, 'THL': {},
+                          'THU': {}, 'UZR': {}, 'STOP': {}}
 
         # write auto constants to string
         cgen = FortranGen()
@@ -525,17 +526,17 @@ class FortranBackend(NumpyBackend):
 
         # write auto constants to file
         try:
-            with open('c.ivp', 'wt') as cfile:
+            with open(f'{directory}/c.ivp', 'wt') as cfile:
                 cfile.write(cgen.generate())
         except FileNotFoundError:
-            with open('c.ivp', 'xt') as cfile:
+            with open(f'{directory}/c.ivp', 'xt') as cfile:
                 cfile.write(cgen.generate())
 
         self._auto_files_generated = True
 
         return fn
 
-    def to_pyauto(self, directory=None, generate_auto_def=True):
+    def to_pyauto(self, directory=None, generate_auto_def=True, **kwargs):
         """
 
         Parameters
@@ -547,11 +548,20 @@ class FortranBackend(NumpyBackend):
         -------
 
         """
-        from pyrates.utility import PyAuto
+        from pyrates.utility.pyauto import PyAuto
         directory = directory if directory else self._build_dir
         if generate_auto_def:
             self.generate_auto_def(directory)
-        return PyAuto(directory)
+        return PyAuto(directory, **kwargs)
+
+    def clear(self) -> None:
+        """Removes all layers, variables and operations from graph. Deletes build directory.
+        """
+
+        wdir = super().clear()
+        for f in [f for f in os.listdir(wdir)
+                  if "cpython" in f and ("rhs_func" in f or "pyrates_func" in f) and f[-3:] == ".so" ]:
+            os.remove(f"{wdir}/{f}")
 
     def _solve(self, rhs_func, func_args, T, dt, dts, t, solver, output_indices, **kwargs):
         """
@@ -576,8 +586,8 @@ class FortranBackend(NumpyBackend):
 
         if self.pyauto_compat:
 
-            from pyrates.utility import PyAuto
-            pyauto = PyAuto(auto_dir=self._build_dir)
+            from pyrates.utility.pyauto import PyAuto
+            pyauto = PyAuto(working_dir=self._build_dir)
             dsmin = dt*1e-2
             auto_defs = {'DSMIN': dsmin, 'DSMAX': dt*1e2, 'NMX': int(T/dsmin), 'NPR': int(dts/dt)}
             for key, val in auto_defs.items():
