@@ -59,7 +59,7 @@ __status__ = "development"
 def grid_search(circuit_template: Union[CircuitTemplate, str], param_grid: Union[dict, pd.DataFrame], param_map: dict,
                 step_size: float, simulation_time: float, inputs: dict, outputs: dict,
                 sampling_step_size: Optional[float] = None, permute_grid: bool = False, init_kwargs: dict = None,
-                **kwargs) -> tuple:
+                clear: bool = True, **kwargs) -> tuple:
     """Function that runs multiple parametrizations of the same circuit in parallel and returns a combined output.
 
     Parameters
@@ -83,6 +83,8 @@ def grid_search(circuit_template: Union[CircuitTemplate, str], param_grid: Union
     permute_grid
         If true, all combinations of the provided param_grid values will be realized. If false, the param_grid values
         will be traversed pairwise.
+    clear
+        If true, all files that have been created by PyRates to run the grid-search will be cleaned up afterwards.
     kwargs
         Additional keyword arguments passed to the `:class:ComputeGraph` initialization.
 
@@ -100,7 +102,7 @@ def grid_search(circuit_template: Union[CircuitTemplate, str], param_grid: Union
 
     if not init_kwargs:
         init_kwargs = {}
-    vectorization = init_kwargs.pop('vectorization', 'nodes')
+    vectorization = init_kwargs.pop('vectorization', True)
     if type(circuit_template) is str:
         circuit_template = CircuitTemplate.from_yaml(circuit_template)
 
@@ -122,9 +124,8 @@ def grid_search(circuit_template: Union[CircuitTemplate, str], param_grid: Union
         new_params = {}
         for key in param_keys:
             new_params[key] = param_grid[key][idx]
-        circuit_tmp = deepcopy(circuit_template).apply()
-        circuit_key = f'{circuit_tmp.label}_{idx}'
-        circuit_tmp = adapt_circuit(circuit_tmp, new_params, param_map)
+        circuit_key = f'{circuit_template.label}_{idx}'
+        circuit_tmp = adapt_circuit(deepcopy(circuit_template).apply(), new_params, param_map)
         circuit.add_circuit(circuit_key, circuit_tmp)
         circuit_names.append(circuit_key)
     param_grid.index = circuit_names
@@ -148,7 +149,10 @@ def grid_search(circuit_template: Union[CircuitTemplate, str], param_grid: Union
                       inputs=inputs,
                       outputs=outputs,
                       **kwargs)    # type: pd.DataFrame
-    #net.clear()
+
+    # clean up config files
+    if clear:
+        net.clear()
 
     # return results
     if 'profile' in kwargs:
@@ -1124,8 +1128,9 @@ class ClusterGridSearch(ClusterCompute):
 
     @staticmethod
     def create_cgs_config(fp: str, circuit_template: str, param_map: dict, dt: float, simulation_time: float,
-                          sampling_step_size: Optional[float], inputs: Optional[dict] = {},
-                          outputs: Optional[dict] = {}, gs_kwargs: Optional[dict] = {}, worker_kwargs: Optional[dict] = {}):
+                          sampling_step_size: Optional[float] = None, inputs: Optional[dict] = {},
+                          outputs: Optional[dict] = {}, gs_kwargs: Optional[dict] = {}, 
+                          worker_kwargs: Optional[dict] = {}):
         """Creates a configfile.json containing a dictionary with all input parameters as key-value pairs
 
         Parameters
@@ -1148,7 +1153,7 @@ class ClusterGridSearch(ClusterCompute):
 
         import yaml
 
-        if not sampling_step_size:
+        if sampling_step_size is None:
             sampling_step_size = dt
 
         config_dict = {
@@ -1259,6 +1264,7 @@ def adapt_circuit(circuit: CircuitIR, params: dict, param_map: dict) -> CircuitI
             # change variable values on nodes
             nodes = param_map[key]['nodes'] if 'nodes' in param_map[key] else []
             for node in nodes:
+                circuit[node].values = deepcopy(circuit[node].values)
                 if "/" in var:
                     op, var_name = var.split("/")
                     if op in circuit[node]:
@@ -1268,7 +1274,7 @@ def adapt_circuit(circuit: CircuitIR, params: dict, param_map: dict) -> CircuitI
                         try:
                             circuit[node].values[op][var] = float(val)
                         except KeyError:
-                            pass
+                            print(f'WARNING: Variable {var} has not been found on node {node}.')
 
             # change variable values on edges
             edges = param_map[key]['edges'] if 'edges' in param_map[key] else []
@@ -1419,13 +1425,10 @@ class ClusterWorkerTemplate(object):
         param_map = global_config_dict['param_map']
         dt = global_config_dict['step_size']
         simulation_time = global_config_dict['simulation_time']
+        sampling_step_size = global_config_dict['sampling_step_size']
 
         # Optional parameters
         #####################
-        try:
-            sampling_step_size = global_config_dict['sampling_step_size']
-        except KeyError:
-            sampling_step_size = dt
 
         try:
             inputs = global_config_dict['inputs']
@@ -1489,7 +1492,7 @@ class ClusterWorkerTemplate(object):
                             permute_grid=False,
                             inputs=inputs,
                             outputs=outputs.copy(),
-                            profile="t",
+                            profile=True,
                             **gs_kwargs)
 
         print(f'Total parameter grid computation time: {t_:.3f} seconds')
@@ -1591,7 +1594,6 @@ class ClusterWorkerTemplate(object):
         for idx, data in self.results.iteritems():
 
             # Add post customized post processing of 'data' here
-
             self.processed_results.loc[:, idx] = data
 
     def worker_test(self):
