@@ -33,6 +33,7 @@
 import pandas as pd
 import numpy as np
 import matplotlib.pyplot as plt
+from matplotlib.ticker import MaxNLocator
 from typing import Union, Optional
 
 # pyrates internal imports
@@ -115,21 +116,21 @@ def create_cmap(name: str = None, palette_type: str = None, as_cmap: bool = True
 
         # create seaborn colormap
         if palette_type == 'cubehelix':
-            cmap = cubehelix_palette(as_cmap=as_cmap, **kwargs)
+            cmap = cubehelix_palette(name, as_cmap=as_cmap, **kwargs)
         elif palette_type == 'dark':
-            cmap = dark_palette(as_cmap=as_cmap, **kwargs)
+            cmap = dark_palette(name, as_cmap=as_cmap, **kwargs)
         elif palette_type == 'light':
-            cmap = light_palette(as_cmap=as_cmap, **kwargs)
+            cmap = light_palette(name, as_cmap=as_cmap, **kwargs)
         elif palette_type == 'hls':
-            cmap = hls_palette(**kwargs)
+            cmap = hls_palette(name, **kwargs)
         elif palette_type == 'husl':
-            cmap = husl_palette(**kwargs)
+            cmap = husl_palette(name, **kwargs)
         elif palette_type == 'diverging':
-            cmap = diverging_palette(as_cmap=as_cmap, **kwargs)
+            cmap = diverging_palette(name, as_cmap=as_cmap, **kwargs)
         elif palette_type == 'crayon':
-            cmap = crayon_palette(**kwargs)
+            cmap = crayon_palette(name, **kwargs)
         elif palette_type == 'xkcd':
-            cmap = xkcd_palette(**kwargs)
+            cmap = xkcd_palette(name, **kwargs)
         elif palette_type == 'mpl':
             cmap = mpl_palette(name, **kwargs)
         else:
@@ -168,7 +169,8 @@ def plot_timeseries(data: pd.DataFrame, variable: str = 'value', plot_style: str
     """
     import seaborn as sb
 
-    sb.set_style(bg_style)
+    if bg_style:
+        sb.set_style(bg_style)
 
     # pre-process data
     demean = kwargs.pop('demean', False)
@@ -339,7 +341,7 @@ def plot_connectivity(fc: Union[np.ndarray, pd.DataFrame], threshold: Optional[f
     # turn fc into dataframe if necessary
     if type(fc) is np.ndarray:
         rows = kwargs.pop('yticklabels') if 'yticklabels' in kwargs.keys() else [str(i) for i in range(fc.shape[0])]
-        cols = kwargs.pop('xticklabels') if 'xticklabels' in kwargs.keys() else [str(i) for i in range(fc.shape[0])]
+        cols = kwargs.pop('xticklabels') if 'xticklabels' in kwargs.keys() else [str(i) for i in range(fc.shape[1])]
         fc = pd.DataFrame(fc, index=[str(r) for r in rows], columns=[str(c) for c in cols])
 
     # apply threshold
@@ -514,7 +516,7 @@ def plot_psd(data: pd.DataFrame, fmin: float = 0., fmax: float = 100., tmin: flo
 
     """
 
-    from pyrates.utility import mne_from_dataframe
+    from pyrates.utility.mne_wrapper import mne_from_dataframe
     from mne.viz import plot_raw_psd
 
     if type(data) is pd.DataFrame and 'out_var' in data.columns.names and len(data.columns.names) > 1:
@@ -802,7 +804,7 @@ def _draw_heatmap(*args, **kwargs):
 
 class Interactive2DParamPlot(object):
     def __init__(self, data_map: np.array, data_series: pd.DataFrame, x_values: np.array, y_values: np.array,
-                 param_map: pd.DataFrame, tmin=0., **kwargs):
+                 x_key: str, y_key: str, param_map: pd.DataFrame, tmin=0., title=None, **kwargs):
         """Creates an interactive 2D plot that allows visualization of time series using button press events
 
         Derive child class and change get_data() respectively to utilize this plotting method
@@ -812,16 +814,20 @@ class Interactive2DParamPlot(object):
         data_map
             2D ndarray containing a value based on each column data_series, respectively.
         data_series
-            DataFrame containing all data series used to crate the data map
+            DataFrame containing all data series used to create the data map
         x_values
             ndarray containing values of data-map columns.
         y_values
             ndarray containing values of data-map rows.
+        x_key
+        y_key
         param_map
             Dataframe containing the mapping between data-series columns (index) and x/y value pairs (columns)
             (as returned by `grid_search`).
         tmin
             Starting point for time-series plots in time units (float).
+        title
+            Title of 2D plot.
         kwargs
             Additional information to access a column in data_series if necessary
 
@@ -830,6 +836,7 @@ class Interactive2DParamPlot(object):
 
         """
         dt = kwargs.pop('step_size', data_series.index[1] - data_series.index[0])
+        state_var = kwargs.pop('state_var', '')
         tmin = int(tmin/dt)
         self.data = data_series.iloc[tmin:, :]
         self.x_values = x_values
@@ -838,11 +845,16 @@ class Interactive2DParamPlot(object):
 
         # set up param map matrix
         self.map = np.zeros(shape=data_map.shape, dtype=np.int32)
-        for i in range(param_map.shape[0]):
-            x, y = param_map.iloc[i, 0], param_map.iloc[i, 1]
-            x_idx = np.argwhere(self.x_values == x)[0]
-            y_idx = np.argwhere(self.y_values == y)[0]
-            self.map[y_idx, x_idx] = i
+        for key in param_map.index:
+            x, y = param_map.loc[key, x_key], param_map.loc[key, y_key]
+            x_idx = np.argmin(np.abs(self.x_values - x))
+            y_idx = np.argmin(np.abs(self.y_values - y))
+            data_idx = data_series.columns.get_loc(key)
+            if type(data_idx) is np.ndarray:
+                data_idx = np.argwhere(data_idx)[0]
+            elif type(data_idx) is not int:
+                data_idx = data_idx.start
+            self.map[y_idx, x_idx] = data_idx
 
         # Create canvas
         if 'subplots' in kwargs:
@@ -854,12 +866,18 @@ class Interactive2DParamPlot(object):
         self.marker = self.ax[0].plot(0, 0, 'x', color='white', markersize='10')
 
         # Plot 2D data in left subplot
-        num_x_ticks = kwargs.pop('num_x_ticks', 10)
-        num_y_ticks = kwargs.pop('num_y_ticks', 10)
-        plot_connectivity(data_map, ax=self.ax[0], yticklabels=list(np.round(y_values, decimals=2)),
-                          xticklabels=list(np.round(x_values, decimals=2)), **kwargs)
-        set_num_axis_ticks(ax=self.ax[0], num_x_ticks_old=data_map.shape[1], num_y_ticks_old=data_map.shape[0],
-                           num_x_ticks_new=num_x_ticks, num_y_ticks_new=num_y_ticks)
+        num_x_ticks = kwargs.pop('num_x_ticks', len(x_values))
+        num_y_ticks = kwargs.pop('num_y_ticks', len(y_values))
+        shrink = kwargs.pop('cbar_shrink', 0.5)
+        im = self.ax[0].imshow(data_map, **kwargs)
+        set_num_axis_ticks(ax=self.ax[0], num_x_ticks=num_x_ticks, num_y_ticks=num_y_ticks)
+        self.ax[0].set_xlabel(x_key)
+        self.ax[0].set_ylabel(y_key)
+        self.ax[0].set_xticklabels([""] + list(np.round(x_values, decimals=2)))
+        self.ax[0].set_yticklabels([""] + list(np.round(y_values, decimals=2)))
+        if title:
+            self.ax[0].set_title(title)
+        plt.colorbar(im, ax=self.ax[0], shrink=shrink)
 
         # set up grid in right subplot
         self.ax[1].grid(visible=True, color="silver")
@@ -868,7 +886,8 @@ class Interactive2DParamPlot(object):
         data_min, data_max = np.min(self.data.values), np.max(self.data.values)
         data_margin = (data_max - data_min) * 0.1
         cmap = create_cmap('pyrates_purple', as_cmap=False, n_colors=1, reverse=True)
-        plot_timeseries(time_series, ax=self.ax[1], ylim=[data_min-data_margin, data_max+data_margin], cmap=cmap)
+        plot_timeseries(time_series, ax=self.ax[1], ylim=[data_min-data_margin, data_max+data_margin], cmap=cmap,
+                        ylabel=state_var)
         self.ax[1].set_title(f'x: {np.round(x, decimals=2)}, y: {np.round(y, decimals=2)}')
 
         # Call Interactive2DPlot class instance when mouse button is pressed inside the 2D plot
@@ -900,7 +919,7 @@ class Interactive2DParamPlot(object):
         self.marker = self.ax[0].plot(x_sample, y_sample, 'x', color='white', markersize='10')
 
         # Update serial plot
-        self.update_lineplot(int(x_sample), int(y_sample))
+        self.update_lineplot(int(np.round(x_sample, decimals=0)), int(np.round(y_sample, decimals=0)))
         self.fig.canvas.draw()
         self.fig.canvas.flush_events()
 
@@ -975,25 +994,18 @@ def load_fig_from_pickle():
         pass
 
 
-def set_num_axis_ticks(ax, num_x_ticks_old, num_y_ticks_old, num_x_ticks_new=10, num_y_ticks_new=10):
+def set_num_axis_ticks(ax, num_x_ticks=10, num_y_ticks=10):
     """Set the number of x and y ticks of a plot axis
 
     Parameters
     ----------
     ax
-    num_x_ticks_old
-    num_y_ticks_old
-    num_x_ticks_new
-    num_y_ticks_new
+    num_x_ticks
+    num_y_ticks
 
     Returns
     -------
 
     """
-    step_tick_x, step_tick_y = int(num_x_ticks_old / num_x_ticks_new), int(num_y_ticks_old / num_y_ticks_new)
-    for n, tick in enumerate(ax.xaxis.iter_ticks()):
-        if n % step_tick_x != 0:
-            tick[0].set_visible(False)
-    for n, tick in enumerate(ax.yaxis.iter_ticks()):
-        if n % step_tick_y != 0:
-            tick[0].set_visible(False)
+    ax.xaxis.set_major_locator(MaxNLocator(num_x_ticks))
+    ax.yaxis.set_major_locator(MaxNLocator(num_y_ticks))
